#include <iostream>
#include <fstream>
using namespace std ;

#include <qfiledialog.h>
#include <qpainter.h>
#include <qmessagebox.h>
#include <qapplication.h>

#include "constants.h"
#include "planche.h"
#include "setIntMath.h"
#include "setDoubleMath.h"
#include "setMandelbrot.h"


//name = xyzwvu  where x,y,z,u,v,w = char 0,1,2,...,9,A,B,...,F
void string2RGB( char* name, long* r, long* g, long*b )
{
 long l,m;

 // Red
 l = name[0]-'0' ;
 if ((l<0) || (l>9))
	{
	 l = name[0] - 'A' ;
	 if ((l>=0) && (l<6)) l += 10 ; else l = 0 ;
	}
 l *= 16 ;
 m = name[1]-'0' ;
 if ((m<0) || (m>9))
	{
	 m = name[1] - 'A' ;
	 if ((m>=0) && (m<6)) m += 10 ; else m = 0 ;
	}
 *r = l+m ;

  // Green
 l = name[2]-'0' ;
 if ((l<0) || (l>9))
	{
	 l = name[2] - 'A' ;
	 if ((l>=0) && (l<6)) l += 10 ; else l = 0 ;
	}
 l *= 16 ;
 m = name[3]-'0' ;
 if ((m<0) || (m>9))
	{
	 m = name[3] - 'A' ;
	 if ((m>=0) && (m<6)) m += 10 ; else m = 0 ;
	}
 *g = l+m ;

 // Blue
 l = name[4]-'0' ;
 if ((l<0) || (l>9))
	{
	 l = name[4] - 'A' ;
	 if ((l>=0) && (l<6)) l += 10 ; else l = 0 ;
	}
 l *= 16 ;
 m = name[5]-'0' ;
 if ((m<0) || (m>9))
	{
	 m = name[5] - 'A' ;
	 if ((m>=0) && (m<6)) m += 10 ; else m = 0 ;
	}
 *b = l+m ;

}


planche :: planche( QWidget* parent=0,const char* name=0, WFlags f=0 ) : QWidget(parent,name,f)
{
 long k ;
 unsigned temp, temp2 ;

 setBackgroundMode( NoBackground ) ;
 sizeX = SIZE_PLANCHE ;
 resize( QSize(SIZE_PLANCHE,SIZE_PLANCHE) );
 _buffer.resize( QSize(SIZE_PLANCHE,SIZE_PLANCHE) ) ;
 bitBlt( this , 0 , 0 , &_buffer ) ;
 //_source = convertToImage () ;

 paletteRGB = new unsigned long[1786] ;
 nbColors = 1786 ;
 for( k=0 ; k < 256 ; k++ )
 	{
	 temp = 255 ;
	 temp <<= 16 ;
	 temp2 = 255-k ;
	 temp2 <<= 8 ;
	 temp += temp2 ;
	 temp += 255 ;
	 paletteRGB[k] = temp ;
	}

  for( k=1 ; k < 256 ; k++ )
 	{
	 temp = 255-k ;
	 temp <<= 16 ;
	 temp += 255 ;
	 paletteRGB[k+255] = temp ;
	 temp = k ;
	 temp <<= 8 ;
	 temp += 255 ;
	 paletteRGB[k+510] = temp ;
	 temp = 255 ;
	 temp <<= 8 ;
	 temp += (255-k) ;
	 paletteRGB[k+765] = temp ;
	 temp = k ;
	 temp <<= 16 ;
	 temp2 = 255 ;
	 temp2 <<= 8 ;
	 temp += temp2 ;
	 paletteRGB[k+1020] = temp ;
	 temp = 255 ;
	 temp <<= 16 ;
	 temp2 = 255-k ;
	 temp2 <<= 8 ;
	 temp += temp2 ;
	 paletteRGB[k+1275] = temp ;
	 temp = 255-k ;
	 temp <<= 16 ;
	 paletteRGB[k+1530] = temp ;
	}

 matrixLong = new long[sizeX*sizeX] ;
 matrixDouble = NULL ;
 xMin = -2.5 ;
 xMax = 1.5 ;
 yMin = -1.5 ;
 yMax = 1.5 ;
 xMinNew = xMin ;
 yMinNew = yMin ;
 xMaxNew = xMax ;
 yMaxNew = yMax ;
 dx = (xMax-xMin)/sizeX ;
 dy = (yMax-yMin)/sizeX ;

 _mandelbrot = new setMandelbrot ;

 calculate() ;
 calculateMiniMaxi() ;

 drawMatrixLong(0,1500,0,0,false) ;
 calculateMiniMaxi() ;
 calculateRepartitionInt(repartitionGlobal,mini,maxi ) ;



 frameCreated  = FALSE ;
 overallMvt    = FALSE ;
 typeButtonPressed = NoButton ;
 frameChanged  = -1 ;

 //show() ;
 emit signalRepartitionIntChanged(repartitionGlobal,mini,maxi) ;
}

planche :: ~planche()
{
 delete paletteRGB ;
 delete _mandelbrot ;
 if (matrixLong!=NULL) delete matrixLong ;
 if (matrixDouble!=NULL) delete matrixDouble ;
}

void planche :: calculateMiniMaxi()
{
 long i,j,minl,maxl,a ;

 minl = matrixLong[0] ;
 maxl = matrixLong[0] ;

  for(i = 0 ; i < sizeX ; i++)
	 for( j=0 ; j < sizeX ; j++ )
	 	{
		 a = matrixLong[i*sizeX+j] ;
		 if (a>maxl) maxl = a ;
		 if (a<minl) minl = a ;
		}

 mini = minl ;
 maxi = maxl ;
}

void planche :: calculateMindMaxd()
{
 double minl,maxl,a ;
 long i,j ;

 minl = matrixDouble[0] ;
 maxl = matrixDouble[0] ;

  for(i = 0 ; i < sizeX ; i++)
	 for( j=0 ; j < sizeX ; j++ )
	 	{
		 a = matrixDouble[i*sizeX+j] ;
		 if (a>maxl) maxl = a ;
		 if (a<minl) minl = a ;
		}

 mind = minl ;
 maxd = maxl ;
}

// calcul la rpartition des valeurs de la matrice matrixInt
// entre les valeurs a et b et stocke le rsulat dans t[SIZE_IMAGE_REPARTITION_X].
void planche :: calculateRepartitionInt(long*t,long a,long b)
{
 long ta[SIZE_IMAGE_REPARTITION_X+1] ;
 long k,i,aux,s ;

 for(i=0;i<=SIZE_IMAGE_REPARTITION_X;i++) ta[i] = a + (i * (b-a))/SIZE_IMAGE_REPARTITION_X ;

 for(i=0;i<SIZE_IMAGE_REPARTITION_X;i++) t[i] = 0 ;
 s = sizeX*sizeX ;
 for(k=0;k<s;k++)
	{
	 aux = matrixLong[k] ;
	 for(i=0;i<SIZE_IMAGE_REPARTITION_X-1;i++)
		{
		 if ((aux>=ta[i]) && (aux<ta[i+1]))
			{
			 t[i]++ ;
			 break ;
			}
		}
	 if ((aux>=ta[SIZE_IMAGE_REPARTITION_X-1]) && (aux<=ta[SIZE_IMAGE_REPARTITION_X]))
			 t[SIZE_IMAGE_REPARTITION_X-1]++ ;
	}
}

// calcul la rpartition des valeurs de la matrice matriceDouble
// entre les valeurs a et b et stocke le rsulat dans t[SIZE_IMAGE_REPARTITION_X].
void planche :: calculateRepartitionDouble(long* t,double a,double b)
{
 double ta[SIZE_IMAGE_REPARTITION_X+1] ;
 double aux ;
 long k,i,s ;

 for(i=0;i<=SIZE_IMAGE_REPARTITION_X;i++) ta[i] = a + (i * (b-a))/SIZE_IMAGE_REPARTITION_X ;

 for(i=0;i<SIZE_IMAGE_REPARTITION_X;i++) t[i] = 0 ;
 s = sizeX*sizeX ;
 for(k=0;k<s;k++)
	{
	 aux = matrixLong[k] ;
	 for(i=0;i<SIZE_IMAGE_REPARTITION_X-1;i++)
		{
		 if ((aux>=ta[i]) && (aux<ta[i+1]))
			{
			 t[i]++ ;
			 break ;
			}
		}
	 if ((aux>=ta[SIZE_IMAGE_REPARTITION_X-1]) && (aux<=ta[SIZE_IMAGE_REPARTITION_X]))
			 t[SIZE_IMAGE_REPARTITION_X-1]++ ;
	}
}

void planche :: setFrame(double xmi,double xma, double ymi, double yma )
{
}


void planche :: calculate()
{
 if (matrixLong!=NULL)
	{
	 _mandelbrot->calculate(matrixLong,xMin,yMax,dx,dy,sizeX) ;
	}
 if (matrixDouble!=NULL)
	{
	}

}


void planche :: drawMatrixLong(long imr,long iMr,long typeF,int del,bool isInverse)
{
 miniReal = imr ;
 maxiReal = iMr ;
 typeFA = typeF ;
 delay = del ;
 isInversed = isInverse ;
 drawMatrix() ;
}

void planche :: drawMatrixDouble(double dmr,double dMr,long typeF,int del,bool isInverse)
{
}

void planche :: drawMatrixLongOnlyDelay(int d)
{
 delay = d ;
 drawMatrix() ;
}

void planche :: drawMatrix()
{
 QPainter windowpainter ;
 QPainter bufferpainter ;
 long i ,j , ic,sizeI,r,g,b,a,nbc,ic2 ;

 windowpainter.begin(this) ;
 bufferpainter.begin( &_buffer ) ;

 if (matrixLong!=NULL)
	{
 	 if (not(isInversed))
		{
		 sizeI = maxiReal-miniReal ;
		 if (sizeI!=0)
			{
			 nbc = nbColors - 1 ;
			 switch(typeFA)
			 	{
			 	 case 0 : for( i = 0 ; i < sizeX ; i++ )
	 					for( j = 0 ; j < sizeX ; j++ )
							{
							 a = matrixLong[i*sizeX+j] ;
							 ic = (a -miniReal)*nbc/sizeI  ;
							 if (ic<0) ic = 0 ;
							 if (ic>nbc) ic = nbc ;
							 ic += nbColors * delay / 100 ;
							 if (ic>nbc) ic -= nbColors ;
							 a = paletteRGB[ic] ;
							 r = a & MASK_COLOR ;
							 g = (a>> 8) & MASK_COLOR ;
							 b = (a>> 16) & MASK_COLOR ;
							 bufferpainter.setPen( QColor(r,g,b) ) ;
							 bufferpainter.drawPoint(j,i) ;
							 windowpainter.setPen( QColor(r,g,b) ) ;
							 windowpainter.drawPoint(j,i) ;
							}
					 break ;
			 	 case 1 : for( i = 0 ; i < sizeX ; i++ )
	 					for( j = 0 ; j < sizeX ; j++ )
							{
							 a = matrixLong[i*sizeX+j] ;
							 ic = (a -miniReal)*nbc/sizeI  ;
							 ic += nbColors * delay / 100 ;
							 ic = ic % nbColors ;
							 if (ic<0) ic += nbColors ;
							 a = paletteRGB[ic] ;
							 r = a & MASK_COLOR ;
							 g = (a>> 8) & MASK_COLOR ;
							 b = (a>> 16) & MASK_COLOR ;
							 bufferpainter.setPen( QColor(r,g,b) ) ;
							 bufferpainter.drawPoint(j,i) ;
							 windowpainter.setPen( QColor(r,g,b) ) ;
							 windowpainter.drawPoint(j,i) ;
							}
					 break ;
			 	 case 2 : for( i = 0 ; i < sizeX ; i++ )
	 					for( j = 0 ; j < sizeX ; j++ )
							{
							 a = matrixLong[i*sizeX+j] ;
							 ic = (a -miniReal)*nbc/sizeI  ;
							 ic2 = ic +nbColors * delay / 100 ;
							 ic = ic2 % nbColors ;
							 ic2 /= nbColors ;
							 if (ic<0) { ic += nbColors ; ic2-- ; }
							 if ((ic2 & 1)==1) ic = nbc - ic ;
							 a = paletteRGB[ic] ;
							 r = a & MASK_COLOR ;
							 g = (a>> 8) & MASK_COLOR ;
							 b = (a>> 16) & MASK_COLOR ;
							 bufferpainter.setPen( QColor(r,g,b) ) ;
							 bufferpainter.drawPoint(j,i) ;
							 windowpainter.setPen( QColor(r,g,b) ) ;
							 windowpainter.drawPoint(j,i) ;
							}
					 break ;
				}
			}
			else
			{
			 a = paletteRGB[0] ;
			 r = a & MASK_COLOR ;
			 g = (a>> 8) & MASK_COLOR ;
			 b = (a>> 16) & MASK_COLOR ;
			 for( i = 0 ; i < sizeX ; i++ )
	 			for( j = 0 ; j < sizeX ; j++ )
					{
					 bufferpainter.setPen( QColor(r,g,b) ) ;
					 bufferpainter.drawPoint(j,i) ;
					 windowpainter.setPen( QColor(r,g,b) ) ;
					 windowpainter.drawPoint(j,i) ;
					}
			}
		}
		else
		{
		 sizeI = maxiReal-miniReal ;
		 if (sizeI!=0)
			{
			 nbc = nbColors - 1 ;
			 switch(typeFA)
			 	{
			 	 case 0 : for( i = 0 ; i < sizeX ; i++ )
	 					for( j = 0 ; j < sizeX ; j++ )
							{
							 a = matrixLong[i*sizeX+j] ;
							 ic = (a -miniReal)*nbc/sizeI  ;
							 if (ic<0) ic = 0 ;
							 if (ic>nbc) ic = nbc ;
							 ic += nbColors * delay / 100 ;
							 if (ic>nbc) ic -= nbColors ;
							 ic = nbc -ic ;
							 a = paletteRGB[ic] ;
							 r = a & MASK_COLOR ;
							 g = (a>> 8) & MASK_COLOR ;
							 b = (a>> 16) & MASK_COLOR ;
							 bufferpainter.setPen( QColor(r,g,b) ) ;
							 bufferpainter.drawPoint(j,i) ;
							 windowpainter.setPen( QColor(r,g,b) ) ;
							 windowpainter.drawPoint(j,i) ;
							}
					 break ;
			 	 case 1 : for( i = 0 ; i < sizeX ; i++ )
	 					for( j = 0 ; j < sizeX ; j++ )
							{
							 a = matrixLong[i*sizeX+j] ;
							 ic = (a -miniReal)*nbc/sizeI  ;
							 ic += nbColors * delay / 100 ;
							 ic = ic % nbColors ;
							 if (ic<0) ic += nbColors ;
							 ic = nbc -ic ;
							 a = paletteRGB[ic] ;
							 r = a & MASK_COLOR ;
							 g = (a>> 8) & MASK_COLOR ;
							 b = (a>> 16) & MASK_COLOR ;
							 bufferpainter.setPen( QColor(r,g,b) ) ;
							 bufferpainter.drawPoint(j,i) ;
							 windowpainter.setPen( QColor(r,g,b) ) ;
							 windowpainter.drawPoint(j,i) ;
							}
					 break ;
			 	 case 2 : for( i = 0 ; i < sizeX ; i++ )
	 					for( j = 0 ; j < sizeX ; j++ )
							{
							 a = matrixLong[i*sizeX+j] ;
							 ic = (a -miniReal)*nbc/sizeI  ;
							 ic2 = ic +nbColors * delay / 100 ;
							 ic = ic2 % nbColors ;
							 ic2 /= nbColors ;
							 if (ic<0) { ic += nbColors ; ic2-- ; }
							 if ((ic2 & 1)==0) ic = nbc - ic ;
							 a = paletteRGB[ic] ;
							 r = a & MASK_COLOR ;
							 g = (a>> 8) & MASK_COLOR ;
							 b = (a>> 16) & MASK_COLOR ;
							 bufferpainter.setPen( QColor(r,g,b) ) ;
							 bufferpainter.drawPoint(j,i) ;
							 windowpainter.setPen( QColor(r,g,b) ) ;
							 windowpainter.drawPoint(j,i) ;
							}
					 break ;
				}
			}
			else
			{
			 a = paletteRGB[nbColors - 1] ;
			 r = a & MASK_COLOR ;
			 g = (a>> 8) & MASK_COLOR ;
			 b = (a>> 16) & MASK_COLOR ;
			 for( i = 0 ; i < sizeX ; i++ )
	 			for( j = 0 ; j < sizeX ; j++ )
					{
					 bufferpainter.setPen( QColor(r,g,b) ) ;
					 bufferpainter.drawPoint(j,i) ;
					 windowpainter.setPen( QColor(r,g,b) ) ;
					 windowpainter.drawPoint(j,i) ;
					}
			}
		}
	}

 if (matrixDouble!=NULL)
	{
 	 if (not(isInversed))
		{

		}
		else
		{

		}
	}

 bufferpainter.end() ;
 windowpainter.end() ;

}

void planche :: calculateF9(QString* p)
{
 bool ok ;

 xMin = p[0].toDouble(&ok) ;
 if (not(ok)) xMin = 0.01 ;
 xMax = p[1].toDouble(&ok) ;
 if (not(ok)) xMax = 0.99 ;
 yMin = p[2].toDouble(&ok) ;
 if (not(ok)) yMin = 0.01 ;
 yMax = p[3].toDouble(&ok) ;
 if (not(ok)) xMin = 0.99 ;
 _mandelbrot->setParameters(p) ;

 dx = (xMax-xMin)/sizeX ;
 dy = (yMax-yMin)/sizeX ;
 QApplication::setOverrideCursor(Qt::waitCursor) ;
 calculate() ;
 QApplication::restoreOverrideCursor() ;
 if (matrixLong!=NULL)
	{
	 calculateMiniMaxi() ;
	 miniReal = mini ;
	 maxiReal = maxi ;
	 calculateRepartitionInt(repartitionGlobal,mini,maxi ) ;
	 emit signalRepartitionIntChanged(repartitionGlobal,mini,maxi) ;
	}
 if (matrixDouble!=NULL)
	{

	}

 drawMatrix() ;
 frameCreated = FALSE ;
}

void planche :: changeCoordinate(QString* p)
{
 bool ok ;

 xMin = p[0].toDouble(&ok) ;
 if (not(ok)) xMin = 0.01 ;
 xMax = p[1].toDouble(&ok) ;
 if (not(ok)) xMax = 0.99 ;
 yMin = p[2].toDouble(&ok) ;
 if (not(ok)) yMin = 0.01 ;
 yMax = p[3].toDouble(&ok) ;
 if (not(ok)) xMin = 0.99 ;
 xMinNew = xMin ;
 xMaxNew = xMax ;
 yMinNew = yMin ;
 yMaxNew = yMax ;
 dx = (xMax-xMin)/sizeX ;
 dy = (yMax-yMin)/sizeX ;

 if (frameCreated)
	{
	 QPainter windowpainter ;
	 QPainter bufferpainter ;

	 windowpainter.begin( this ) ;
	 bufferpainter.begin( &_buffer ) ;
	 windowpainter.setRasterOp( NotXorROP ) ;
	 bufferpainter.setRasterOp( NotXorROP ) ;
	 windowpainter.setPen( blue) ;
	 bufferpainter.setPen( blue ) ;
	 bufferpainter.drawRect(oldPoint.x(),oldPoint.y(),
	 			newPoint.x() - oldPoint.x(),
				newPoint.y() - oldPoint.y()) ;
	 bufferpainter.drawRect(oldPoint.x()-3,oldPoint.y()-3,6,6) ;
	 bufferpainter.drawRect(oldPoint.x()-3,newPoint.y()-3,6,6) ;
	 bufferpainter.drawRect(newPoint.x()-3,oldPoint.y()-3,6,6) ;
	 bufferpainter.drawRect(newPoint.x()-3,newPoint.y()-3,6,6) ;
	 windowpainter.drawRect(oldPoint.x(),oldPoint.y(),
	 			newPoint.x() - oldPoint.x(),
				newPoint.y() - oldPoint.y()) ;
	 windowpainter.drawRect(oldPoint.x()-3,oldPoint.y()-3,6,6) ;
	 windowpainter.drawRect(oldPoint.x()-3,newPoint.y()-3,6,6) ;
	 windowpainter.drawRect(newPoint.x()-3,oldPoint.y()-3,6,6) ;
	 windowpainter.drawRect(newPoint.x()-3,newPoint.y()-3,6,6) ;
	 windowpainter.end() ;
	 bufferpainter.end() ;
	 frameCreated = false ;
	}
}

void planche :: changeParametersWithoutXY(QString* p)
{
 _mandelbrot->setParameters(p) ;
}

void planche :: changeSizeMatrix(long s)
{
 if (matrixLong!=NULL)
	{
	 delete matrixLong ;
	 matrixLong = new long[s*s] ;
	  if (matrixLong==NULL)
		{
		 matrixLong = new long[128*128] ;
		 sizeX = 128 ;
		}
		else sizeX = s ;
	}
 if (matrixDouble!=NULL)
	{
	 delete matrixDouble ;
	 // faire la suite
	}

 resize( QSize(sizeX,sizeX) );
 _buffer.resize( QSize(sizeX,sizeX) ) ;
}

void planche :: loadPalette(QString f)
{
 long i,r,g,b,nbC ;
 char ch[9] ;

 ifstream entreeText( f.latin1()  , ios::in ) ;

 if (entreeText)
 	{
	 entreeText.getline(ch,8) ;
	 if (not(entreeText.eof()))
		{
		 nbC = atol(ch) ;
		 if ((nbC>0) && (nbC<1000000))
			{
			 delete paletteRGB ;
			 paletteRGB = new unsigned long[nbC] ;
			 nbColors = nbC ;
			 for(i=0;i<nbC;i++)
				{
				 entreeText.getline(ch,8) ;
				 if (entreeText.eof()) break ;
				 string2RGB( ch,&r,&g,&b ) ;
				 g <<= 8 ;
				 b <<= 16 ;
				 r += (g+b) ;
				 paletteRGB[i] = r ;
				}
			  drawMatrix() ;
			}
		}
	}
}

void planche :: saveImage(QString f)
{
 QString nb ;

 if ( !_buffer.save( f+".bmp", "BMP") )
 		QMessageBox::warning(0,"Erreur d'enregistrement" ,
					"impossible d'enregistrer le fichier") ;


 QFile file( f+".txt" ) ;
 if (file.open( IO_WriteOnly ))
	{
	 QTextStream stream( &file ) ;
	 stream << "[Scope] [Precision] [Xmin,Xmax,Ymin,Ymax]" << endl ;
	 stream << "16" << endl ;
	 stream << nb.setNum( xMin,'g',16) << endl ;
	 stream << nb.setNum( xMax,'g',16) << endl ;
	 stream << nb.setNum( yMin,'g',16) << endl ;
	 stream << nb.setNum( yMax,'g',16) << endl ;
	 _mandelbrot->streamParameters(stream) ;

	 file.close() ;
	}

}

void planche :: mouseDoubleClickEvent( QMouseEvent* e)
{
 //QString sXmin, sXmax , sYmin ,sYmax , sIter ;
 //bool ok ;


 if (frameCreated)
	{
	 if (fabs(xMin-xMinNew)+fabs(xMax-xMaxNew)+fabs(yMin-yMinNew)+fabs(yMax-yMaxNew) >= EPSILON)
		{
		 xMin = xMinNew ;
		 xMax = xMaxNew ;
		 yMin = yMinNew ;
		 yMax = yMaxNew ;
		}
	}
	else
	{
	/*
	 _frame->getEdit(&sXmin, &sXmax , &sYmin ,&sYmax) ;
	 xMin = sXmin.toDouble(&ok) ;
	 xMax = sXmax.toDouble(&ok) ;
	 yMin = sYmin.toDouble(&ok) ;
	 yMax = sYmax.toDouble(&ok) ;
	 //cout << xMin << " " << xMax << " : " << yMin << " " << yMax << "\n" ;
	 if (fabs(xMin-coord[currentCoord][0])+
	     fabs(xMax-coord[currentCoord][1])+
	     fabs(yMin-coord[currentCoord][2])+
	     fabs(yMax-coord[currentCoord][3]) >= EPSILON)
		{
		 currentCoord++ ;
		 coord[currentCoord][0] = xMin ;
		 coord[currentCoord][1] = xMax ;
		 coord[currentCoord][2] = yMin ;
		 coord[currentCoord][3] = yMax ;
		}
	 */
	}

 dx = (xMax-xMin)/sizeX ;
 dy = (yMax-yMin)/sizeX ;
 QApplication::setOverrideCursor(Qt::waitCursor) ;
 calculate() ;
 QApplication::restoreOverrideCursor() ;
 if (matrixLong!=NULL)
	{
	 calculateMiniMaxi() ;
	 miniReal = mini ;
	 maxiReal = maxi ;
	 calculateRepartitionInt(repartitionGlobal,mini,maxi ) ;
	 emit signalRepartitionIntChanged(repartitionGlobal,mini,maxi) ;
	}
 if (matrixDouble!=NULL)
	{

	}

 drawMatrix() ;
 frameCreated = FALSE ;

 emit signalFrameChanged(xMin,xMax,yMin,yMax) ;
}


void planche :: mousePressEvent( QMouseEvent* event )
{
 long XXinf , XXsup , YYinf , YYsup , X , Y ;

 if ( oldPoint.x() < newPoint.x() )
 	{
	 XXinf = oldPoint.x() ;
	 XXsup = newPoint.x() ;
	}
	else
	{
	 XXinf = newPoint.x() ;
	 XXsup = oldPoint.x() ;
	}
 if ( oldPoint.y() < newPoint.y() )
 	{
	 YYinf = oldPoint.y() ;
	 YYsup = newPoint.y() ;
	}
	else
	{
	 YYinf = newPoint.y() ;
	 YYsup = oldPoint.y() ;
	}
 X = event->x() ;
 Y = event->y() ;

 if (( XXinf <= X ) && ( X <= XXsup ) &&
     ( YYinf <= Y ) && ( Y <= YYsup ))
    overallMvt = TRUE ; else overallMvt = FALSE ;

 if ((event->button() == LeftButton ) && not(frameCreated))
 	{
	 oldPoint = event->pos() ;
	 xMinNew = xMin + dx * X ;
	 yMinNew = yMax - dy * Y ;
	}


 if (( X >= oldPoint.x()-3 ) && ( X <= oldPoint.x()+3 ) &&
     ( Y >= oldPoint.y()-3 ) && ( Y <= oldPoint.y()+3 ) ) frameChanged = 1 ;
 if (( X >= newPoint.x()-3 ) && ( X <= newPoint.x()+3 ) &&
     ( Y >= oldPoint.y()-3 ) && ( Y <= oldPoint.y()+3 ) ) frameChanged = 2 ;
 if (( X >= newPoint.x()-3 ) && ( X <= newPoint.x()+3 ) &&
     ( Y >= newPoint.y()-3 ) && ( Y <= newPoint.y()+3 ) ) frameChanged = 3 ;
 if (( X >= oldPoint.x()-3 ) && ( X <= oldPoint.x()+3 ) &&
     ( Y >= newPoint.y()-3 ) && ( Y <= newPoint.y()+3 ) ) frameChanged = 4 ;

 if ( frameChanged > -1 ) overallMvt = FALSE ;
 mvtPoint = event->pos() ;
 typeButtonPressed = event->button() ;
}


void planche :: mouseReleaseEvent( QMouseEvent* event )
{
 double Aux ;
 long X , Y , Xo , Yo ;

 X = event->x() ;
 Y = event->y() ;
 Xo = oldPoint.x() ;
 Yo = oldPoint.y() ;
 if (( typeButtonPressed == LeftButton ) && not(frameCreated))
 	{
	 if ((X>=Xo-3) && (X<=Xo+3) && (Y>=Yo-3) && (Y<=Yo+3))
	 	{
		 frameCreated = FALSE ;
		}
		else
		{
	 	 newPoint = event->pos() ;
	 	 frameCreated = TRUE ;
		 xMaxNew = xMin + dx * X ;
		 yMaxNew = yMax - dy * Y ;
		 if ( xMinNew > xMaxNew )
			{
			 Aux = xMinNew ;
		 	 xMinNew = xMaxNew ;
		 	 xMaxNew = Aux ;
			}
		 if ( yMinNew > yMaxNew )
			{
		 	 Aux = yMinNew ;
		 	 yMinNew = yMaxNew ;
		 	 yMaxNew = Aux ;
			}
		 //emit signalFrameChanged(xMinNew,xMaxNew,yMinNew,yMaxNew) ;
		}
	}

 frameChanged = -1 ;
 typeButtonPressed = NoButton ;
}


void planche :: mouseMoveEvent( QMouseEvent* event )
{
 QPainter windowpainter ;
 QPainter bufferpainter ;
 long X , Y , ax , ay ;
 //double xAux , yAux ;

 X = event->x() ;
 Y = event->y() ;
 windowpainter.begin( this ) ;
 bufferpainter.begin( &_buffer ) ;
 windowpainter.setRasterOp( NotXorROP ) ;
 bufferpainter.setRasterOp( NotXorROP ) ;
 windowpainter.setPen( blue) ;
 bufferpainter.setPen( blue ) ;

 if ( (typeButtonPressed == LeftButton) && not(frameCreated))
	{
	 bufferpainter.drawRect(oldPoint.x(),oldPoint.y(),
	 			mvtPoint.x() - oldPoint.x(),
				mvtPoint.y() - oldPoint.y()) ;
	 bufferpainter.drawRect(oldPoint.x()-3,oldPoint.y()-3,6,6) ;
	 bufferpainter.drawRect(oldPoint.x()-3,mvtPoint.y()-3,6,6) ;
	 bufferpainter.drawRect(mvtPoint.x()-3,oldPoint.y()-3,6,6) ;
	 bufferpainter.drawRect(mvtPoint.x()-3,mvtPoint.y()-3,6,6) ;
	 windowpainter.drawRect(oldPoint.x(),oldPoint.y(),
	 			mvtPoint.x() - oldPoint.x(),
				mvtPoint.y() - oldPoint.y()) ;
	 windowpainter.drawRect(oldPoint.x()-3,oldPoint.y()-3,6,6) ;
	 windowpainter.drawRect(oldPoint.x()-3,mvtPoint.y()-3,6,6) ;
	 windowpainter.drawRect(mvtPoint.x()-3,oldPoint.y()-3,6,6) ;
	 windowpainter.drawRect(mvtPoint.x()-3,mvtPoint.y()-3,6,6) ;
	 mvtPoint = event->pos() ;
	 bufferpainter.drawRect(oldPoint.x(),oldPoint.y(),
	 			mvtPoint.x() - oldPoint.x(),
				mvtPoint.y() - oldPoint.y()) ;
	 bufferpainter.drawRect(oldPoint.x()-3,oldPoint.y()-3,6,6) ;
	 bufferpainter.drawRect(oldPoint.x()-3,mvtPoint.y()-3,6,6) ;
	 bufferpainter.drawRect(mvtPoint.x()-3,oldPoint.y()-3,6,6) ;
	 bufferpainter.drawRect(mvtPoint.x()-3,mvtPoint.y()-3,6,6) ;
	 windowpainter.drawRect(oldPoint.x(),oldPoint.y(),
	 			mvtPoint.x() - oldPoint.x(),
				mvtPoint.y() - oldPoint.y()) ;
	 windowpainter.drawRect(oldPoint.x()-3,oldPoint.y()-3,6,6) ;
	 windowpainter.drawRect(oldPoint.x()-3,mvtPoint.y()-3,6,6) ;
	 windowpainter.drawRect(mvtPoint.x()-3,oldPoint.y()-3,6,6) ;
	 windowpainter.drawRect(mvtPoint.x()-3,mvtPoint.y()-3,6,6) ;
	 //xMaxNew = xMax + dx * mvtPoint.x() ;
	 //yMaxNew = yMax -  dy * mvtPoint.y() ;
	}
if ((typeButtonPressed == LeftButton) && overallMvt && frameCreated)
	{
	 bufferpainter.drawRect(oldPoint.x(),oldPoint.y(),
	 			newPoint.x() - oldPoint.x(),
				newPoint.y() - oldPoint.y()) ;
	 bufferpainter.drawRect(oldPoint.x()-3,oldPoint.y()-3,6,6) ;
	 bufferpainter.drawRect(oldPoint.x()-3,newPoint.y()-3,6,6) ;
	 bufferpainter.drawRect(newPoint.x()-3,oldPoint.y()-3,6,6) ;
	 bufferpainter.drawRect(newPoint.x()-3,newPoint.y()-3,6,6) ;
	 windowpainter.drawRect(oldPoint.x(),oldPoint.y(),
	 			newPoint.x() - oldPoint.x(),
				newPoint.y() - oldPoint.y()) ;
	 windowpainter.drawRect(oldPoint.x()-3,oldPoint.y()-3,6,6) ;
	 windowpainter.drawRect(oldPoint.x()-3,newPoint.y()-3,6,6) ;
	 windowpainter.drawRect(newPoint.x()-3,oldPoint.y()-3,6,6) ;
	 windowpainter.drawRect(newPoint.x()-3,newPoint.y()-3,6,6) ;
	 ax = X - mvtPoint.x() ;
	 ay = Y - mvtPoint.y() ;
	 mvtPoint = event->pos() ;
	 oldPoint.setX(oldPoint.x() + ax) ;
	 oldPoint.setY(oldPoint.y() + ay) ;
	 newPoint.setX(newPoint.x() + ax) ;
	 newPoint.setY(newPoint.y() + ay) ;
	 bufferpainter.drawRect(oldPoint.x(),oldPoint.y(),
	 			newPoint.x() - oldPoint.x(),
				newPoint.y() - oldPoint.y()) ;
	 bufferpainter.drawRect(oldPoint.x()-3,oldPoint.y()-3,6,6) ;
	 bufferpainter.drawRect(oldPoint.x()-3,newPoint.y()-3,6,6) ;
	 bufferpainter.drawRect(newPoint.x()-3,oldPoint.y()-3,6,6) ;
	 bufferpainter.drawRect(newPoint.x()-3,newPoint.y()-3,6,6) ;
	 windowpainter.drawRect(oldPoint.x(),oldPoint.y(),
	 			newPoint.x() - oldPoint.x(),
				newPoint.y() - oldPoint.y()) ;
	 windowpainter.drawRect(oldPoint.x()-3,oldPoint.y()-3,6,6) ;
	 windowpainter.drawRect(oldPoint.x()-3,newPoint.y()-3,6,6) ;
	 windowpainter.drawRect(newPoint.x()-3,oldPoint.y()-3,6,6) ;
	 windowpainter.drawRect(newPoint.x()-3,newPoint.y()-3,6,6) ;
	 xMinNew += ax * dx ;
	 yMinNew -= ay * dy ;
	 xMaxNew += ax * dx ;
	 yMaxNew -= ay * dy ;
	 //emit signalFrameChanged(xMinNew,xMaxNew,yMinNew,yMaxNew) ;
	}
	
 if ((typeButtonPressed == LeftButton) && frameCreated && (frameChanged != -1) )
	{
	 bufferpainter.drawRect(oldPoint.x(),oldPoint.y(),
	 			newPoint.x() - oldPoint.x(),
				newPoint.y() - oldPoint.y()) ;
	 bufferpainter.drawRect(oldPoint.x()-3,oldPoint.y()-3,6,6) ;
	 bufferpainter.drawRect(oldPoint.x()-3,newPoint.y()-3,6,6) ;
	 bufferpainter.drawRect(newPoint.x()-3,oldPoint.y()-3,6,6) ;
	 bufferpainter.drawRect(newPoint.x()-3,newPoint.y()-3,6,6) ;
	 windowpainter.drawRect(oldPoint.x(),oldPoint.y(),
	 			newPoint.x() - oldPoint.x(),
				newPoint.y() - oldPoint.y()) ;
	 windowpainter.drawRect(oldPoint.x()-3,oldPoint.y()-3,6,6) ;
	 windowpainter.drawRect(oldPoint.x()-3,newPoint.y()-3,6,6) ;
	 windowpainter.drawRect(newPoint.x()-3,oldPoint.y()-3,6,6) ;
	 windowpainter.drawRect(newPoint.x()-3,newPoint.y()-3,6,6) ;
	 //mvtPoint = event->pos() ; // ????
	 switch (frameChanged)
	 	{
		 case 1 : oldPoint = event->pos() ;
		 	  break ;
		 case 2 : newPoint.setX( X ) ;
		 	  oldPoint.setY( Y ) ;
			  break ;
		 case 3 : newPoint = event->pos() ;
		 	  break ;
		 case 4 : oldPoint.setX( X ) ;
		 	  newPoint.setY( Y ) ;
			  break ;
		}
	 bufferpainter.drawRect(oldPoint.x(),oldPoint.y(),
	 			newPoint.x() - oldPoint.x(),
				newPoint.y() - oldPoint.y()) ;
	 bufferpainter.drawRect(oldPoint.x()-3,oldPoint.y()-3,6,6) ;
	 bufferpainter.drawRect(oldPoint.x()-3,newPoint.y()-3,6,6) ;
	 bufferpainter.drawRect(newPoint.x()-3,oldPoint.y()-3,6,6) ;
	 bufferpainter.drawRect(newPoint.x()-3,newPoint.y()-3,6,6) ;
	 windowpainter.drawRect(oldPoint.x(),oldPoint.y(),
	 			newPoint.x() - oldPoint.x(),
				newPoint.y() - oldPoint.y()) ;
	 windowpainter.drawRect(oldPoint.x()-3,oldPoint.y()-3,6,6) ;
	 windowpainter.drawRect(oldPoint.x()-3,newPoint.y()-3,6,6) ;
	 windowpainter.drawRect(newPoint.x()-3,oldPoint.y()-3,6,6) ;
	 windowpainter.drawRect(newPoint.x()-3,newPoint.y()-3,6,6) ;
	 if (oldPoint.x()< newPoint.x())
		{X = oldPoint.x(); ax = newPoint.x();}
		else
		{X = newPoint.x(); ax = oldPoint.x();}
	 if (oldPoint.y()< newPoint.y())
		{Y = oldPoint.y(); ay = newPoint.y();}
		else
		{Y = newPoint.y(); ay = oldPoint.y();}
	 xMinNew = xMin + dx * X ;
	 yMinNew = yMax - dy * ay ;
	 xMaxNew = xMin + dx * ax ;
	 yMaxNew = yMax - dy * Y ;
	 //emit signalFrameChanged(xMinNew,xMaxNew,yMinNew,yMaxNew) ;
	}

 windowpainter.end() ;
 bufferpainter.end() ;
 //bitBlt( this , 0 , 0 , &_buffer ) ;
}

void planche :: paintEvent( QPaintEvent* event )
{
 bitBlt( this , 0 , 0 , &_buffer ) ;
}

